/*
 * Copyright 2004-2009 the Seasar Foundation and the Others.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND,
 * either express or implied. See the License for the specific language
 * governing permissions and limitations under the License.
 */
package org.seasar.doma.internal.apt.meta;

import static org.seasar.doma.internal.util.AssertionUtil.*;

import java.util.ArrayList;
import java.util.Collections;
import java.util.Deque;
import java.util.Iterator;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import javax.lang.model.element.TypeElement;
import javax.lang.model.type.TypeMirror;

import org.seasar.doma.internal.util.CompositeIterator;

/**
 * @author taedium
 * 
 */
public class EntityMeta {

    protected final Deque<Map<TypeMirror, TypeMirror>> typeParamMapStack = new LinkedList<Map<TypeMirror, TypeMirror>>();

    protected final List<TypeMirror> supertypes = new ArrayList<TypeMirror>();

    protected final List<EntityPropertyMeta> allPropertyMetas = new ArrayList<EntityPropertyMeta>();

    protected final List<EntityDelegateMeta> allDelegateMetas = new ArrayList<EntityDelegateMeta>();

    protected EntityPropertyMeta versionPropertyMeta;

    protected EntityPropertyMeta generatedIdPropertyMeta;

    protected TableMeta tableMeta;

    protected TypeMirror entityType;

    protected TypeElement entityElement;

    protected TypeMirror listenerType;

    protected String name;

    protected boolean mappedSuperclass;

    protected long serialVersionUID;

    protected boolean parametalizedDomain;

    public EntityMeta() {
        typeParamMapStack.push(Collections.<TypeMirror, TypeMirror> emptyMap());
    }

    public TypeMirror getEntityType() {
        return entityType;
    }

    public void setEntityType(TypeMirror entityType) {
        this.entityType = entityType;
    }

    public String getName() {
        return name;
    }

    public void setName(String name) {
        this.name = name;
    }

    public TableMeta getTableMeta() {
        return tableMeta;
    }

    public void setTableMeta(TableMeta tableMeta) {
        this.tableMeta = tableMeta;
    }

    public TypeElement getEntityElement() {
        return entityElement;
    }

    public void setEntityElement(TypeElement entityElement) {
        this.entityElement = entityElement;
    }

    public TypeMirror getListenerType() {
        return listenerType;
    }

    public void setListenerType(TypeMirror listenerType) {
        this.listenerType = listenerType;
    }

    public void addPropertyMeta(EntityPropertyMeta propertyMeta) {
        assertNotNull(propertyMeta);
        allPropertyMetas.add(propertyMeta);
        if (propertyMeta.isParameterizedReturnType()) {
            parametalizedDomain = true;
        }
        if (propertyMeta.isTrnsient()) {
            return;
        }
        if (propertyMeta.isId()) {
            if (propertyMeta.getIdGeneratorMeta() != null) {
                generatedIdPropertyMeta = propertyMeta;
            }
        }
        if (propertyMeta.isVersion()) {
            versionPropertyMeta = propertyMeta;
        }
    }

    public List<EntityPropertyMeta> getAllPropertyMetas() {
        return allPropertyMetas;
    }

    public boolean hasVersionPropertyMeta() {
        return versionPropertyMeta != null;
    }

    public EntityPropertyMeta getVersionPropertyMeta() {
        return versionPropertyMeta;
    }

    public boolean hasGeneratedIdPropertyMeta() {
        return generatedIdPropertyMeta != null;
    }

    public EntityPropertyMeta getGeneratedIdPropertyMeta() {
        return generatedIdPropertyMeta;
    }

    public void addDelegateMeta(EntityDelegateMeta delegateMeta) {
        assertNotNull(delegateMeta);
        allDelegateMetas.add(delegateMeta);
    }

    public List<EntityDelegateMeta> getAllDelegateMetas() {
        return allDelegateMetas;
    }

    public Iterator<? extends EntityMethodMeta> getAllMethodMetaIterator() {
        List<Iterator<? extends EntityMethodMeta>> iterators = new ArrayList<Iterator<? extends EntityMethodMeta>>();
        iterators.add(allPropertyMetas.iterator());
        iterators.add(allDelegateMetas.iterator());
        return new CompositeIterator<EntityMethodMeta>(iterators);
    }

    public void addTypeParameterMap(Map<TypeMirror, TypeMirror> typeParameterMap) {
        typeParamMapStack.push(typeParameterMap);
    }

    public Map<TypeMirror, TypeMirror> getTypeParameterMap() {
        return typeParamMapStack.peek();
    }

    public void addSupertype(TypeMirror supertype) {
        supertypes.add(supertype);
    }

    public List<TypeMirror> getSupertypes() {
        return supertypes;
    }

    public boolean isMappedSuperclass() {
        return mappedSuperclass;
    }

    public void setMappedSuperclass(boolean mappedSuperclass) {
        this.mappedSuperclass = mappedSuperclass;
    }

    public long getSerialVersionUID() {
        return serialVersionUID;
    }

    public void setSerialVersionUID(long serialVersionUID) {
        this.serialVersionUID = serialVersionUID;
    }

    public boolean hasParametalizedDomain() {
        return parametalizedDomain;
    }

}
